#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib
   @package: hspylib.modules.security
      @file: security.py
   @created: Tue, 4 May 2021
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""

from cryptography.fernet import Fernet
from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.kdf.pbkdf2 import PBKDF2HMAC
from hspylib.core.enums.charset import Charset
from hspylib.core.preconditions import check_argument, check_state

import base64
import os

# fmt: off
# !!!Please do not modify the values below!!!
DEFAULT_HS_SALT: str        = "HsPyLib"
DEFAULT_HS_LENGTH: int      = 32
DEFAULT_HS_ITERATIONS: int  = 100000
# fmt: on


def encode_file(in_file: str, out_file: str, binary: bool = False, encoding: str | Charset = Charset.UTF_8) -> int:
    """Encode file into base64
    :param in_file: The file to be encoded
    :param out_file: The resulting encoded file
    :param binary: The file mode text/binary
    :param encoding: The text encoding
    """
    if binary:
        with open(in_file, "rb") as f_in_file:
            with open(out_file, "wb") as f_out_file:
                data = base64.b64encode(f_in_file.read())
                return f_out_file.write(data)

    with open(in_file, "r", encoding=Charset.UTF_8.val) as f_in_file:
        with open(out_file, "w", encoding=str(encoding)) as f_out_file:
            data = base64.b64encode(str.encode(f_in_file.read()))
            return f_out_file.write(str(data, encoding=str(encoding)))


def decode_file(in_file: str, out_file: str, binary: bool = False, encoding: str | Charset = Charset.UTF_8) -> int:
    """Decode file from base64
    :param in_file: The file to be decoded
    :param out_file: The resulting decoded file
    :param binary: The file mode text/binary
    :param encoding: The text encoding
    """
    if binary:
        with open(in_file, "rb") as f_in_file:
            with open(out_file, "wb") as f_out_file:
                data = base64.b64decode(f_in_file.read())
                return f_out_file.write(data)

    with open(in_file, "r", encoding=str(encoding)) as f_in_file:
        with open(out_file, "w", encoding=str(encoding)) as f_out_file:
            data = base64.b64decode(f_in_file.read())
            return f_out_file.write(str(data, encoding=str(encoding)))


def encrypt_file(
    in_file: str,
    out_file: str,
    pass_phrase: str,
    salt: str = DEFAULT_HS_SALT,
    digest_algo: hashes.HashAlgorithm = hashes.SHA256(),
    length: int = DEFAULT_HS_LENGTH,
    iterations: int = DEFAULT_HS_ITERATIONS,
    encoding: str | Charset = Charset.UTF_8,
) -> None:
    """Encrypt file using Fernet cryptography
    :param in_file: The file to be encrypted
    :param out_file: The resulting encrypted file
    :param pass_phrase: The passphrase to encrypt the file
    :param salt: A random data that is used as an additional input to a one-way function to hash data.
    :param digest_algo: The digest encrypting algorithm
    :param length: The desired length of the derived key in bytes. Maximum is (232 - 1) * algorithm.digest_size.
    :param iterations: The number of iterations to perform of the hash function.
    :param encoding: The name of the encoding used to decode or encode the file.
    """

    kdf = PBKDF2HMAC(
        algorithm=digest_algo,
        length=length,
        salt=salt.encode(str(encoding)),
        iterations=iterations,
        backend=default_backend(),
    )
    key = base64.urlsafe_b64encode(kdf.derive(pass_phrase.encode(str(encoding))))
    f = Fernet(key)
    check_argument(os.path.exists(in_file), 'Input file "{}" does not exist', in_file)
    with open(in_file, encoding=str(encoding)) as f_in_file:
        with open(out_file, "w", encoding=str(encoding)) as f_out_file:
            f_out_file.write(f.encrypt(f_in_file.read().encode(str(encoding))).decode(str(encoding)))
    check_state(os.path.exists(out_file), 'Unable to encrypt file "{}"', in_file)


def decrypt_file(
    in_file: str,
    out_file: str,
    pass_phrase: str,
    salt: str = DEFAULT_HS_SALT,
    digest_algo: hashes.HashAlgorithm = hashes.SHA256(),
    length: int = DEFAULT_HS_LENGTH,
    iterations: int = DEFAULT_HS_ITERATIONS,
    encoding: str | Charset = Charset.UTF_8,
) -> None:
    """Decrypt file using Fernet cryptography
    :param in_file: The file to be decrypted
    :param out_file: The resulting decrypted file
    :param pass_phrase: The passphrase to decrypt the file
    :param salt: A random data that is used as an additional input to a one-way function to hash data.DO
    :param digest_algo: The digest decrypting algorithm
    :param length: The desired length of the derived key in bytes. Maximum is (232 - 1) * algorithm.digest_size.
    :param iterations: The number of iterations to perform of the hash function.
    :param encoding: The name of the encoding used to decode or encode the file.
    """

    kdf = PBKDF2HMAC(
        algorithm=digest_algo,
        length=length,
        salt=salt.encode(str(encoding)),
        iterations=iterations,
        backend=default_backend(),
    )
    key = base64.urlsafe_b64encode(kdf.derive(pass_phrase.encode(str(encoding))))
    f = Fernet(key)
    check_argument(os.path.exists(in_file), 'Input file "{}" does not exist', in_file)
    with open(in_file, encoding=str(encoding)) as f_in_file:
        with open(out_file, "w", encoding=str(encoding)) as f_out_file:
            f_out_file.write(f.decrypt(f_in_file.read().encode(str(encoding))).decode(str(encoding)))
    check_state(os.path.exists(out_file), 'Unable to decrypt file "{}"', in_file)


def b64_encode(text: str, encoding: str | Charset = Charset.UTF_8) -> str:
    """Encode a text using Base64.
    :param text The text to be encoded
    :param encoding The name of the encoding used to decode or encode the text.
    """

    return str(base64.b64encode(bytes(text, str(encoding))), str(encoding))


def b64_decode(text: str, encoding: str | Charset = Charset.UTF_8) -> str:
    """Decode a Base64 encoded text.
    :param text The text to be decoded
    :param encoding The name of the encoding used to decode or encode the text.
    """
    return str(base64.b64decode(bytes(text, str(encoding))), str(encoding))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib
   @package: hspylib.core.config
      @file: settings.py
   @created: Tue, 23 Apr 2024
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior
      @site: https://github.com/yorevs/hspylib
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright·(c)·2024,·HSPyLib
"""

import os
from typing import Any

from hspylib.core.config.properties import Properties
from hspylib.core.preconditions import check_argument
from hspylib.core.tools.commons import touch_file
from hspylib.core.tools.text_tools import ensure_endswith


class Settings(Properties):
    """The Settings class represents a persistent set of mutable settings. Each key and its corresponding value in the
    settings dictionary.Format and rules follows the properties approach."""

    def __init__(self, filename: str = None, profile: str | None = None, load_dir: str | None = None) -> None:
        _, ext = os.path.splitext(filename)
        check_argument(ext == ".properties", "Only '.properties' formatted files are permitted!")
        self._load_dir = load_dir
        self._filename = filename
        touch_file(self.filepath)
        super().__init__(filename, profile, load_dir)

    @property
    def filepath(self) -> str:
        return ensure_endswith(f"{self._load_dir}/{self._filename}", ".properties")

    def set(self, key: str, value: Any) -> None:
        """Add or set the setting and it's associated value."""
        self._properties[key] = value

    def remove(self, key: str) -> str:
        """Remove a setting specified by key."""
        return self._properties.pop(key, None)

    def clear(self) -> None:
        """Remove all settings."""
        self._properties.clear()

    def save(self) -> None:
        """Save current settings to the associated file."""
        with open(self.filepath, "w") as f_settings:
            f_settings.writelines([f"{key} = {value} \n" for (key, value) in self._properties.items()])
            f_settings.flush()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
   @project: HsPyLib
   @package: hspylib.core.config
      @file: path_object.py
   @created: Wed, 10 Jan 2024
    @author: <B>H</B>ugo <B>S</B>aporetti <B>J</B>unior"
      @site: https://github.com/yorevs/askai
   @license: MIT - Please refer to <https://opensource.org/licenses/MIT>

   Copyright (c) 2024, HomeSetup
"""

import json
import os
from dataclasses import dataclass
from os.path import expandvars, relpath, join
from pathlib import Path
from typing import Literal, Optional

from hspylib.core.metaclass.classpath import AnyPath


@dataclass(frozen=True)
class PathObject:
    """Dataclass that represents a system path. '~' and system variables are already resolved."""

    abs_dir: str
    rel_dir: str
    filename: str
    exists: bool
    kind: Literal["file", "folder"]

    @staticmethod
    def of(path_name: AnyPath) -> Optional['PathObject']:
        """Create a path object from a any path."""
        if path_name:
            path_dir: Path = Path(path_name) if isinstance(path_name, str) else path_name
            posix_path = Path(expandvars(path_dir.expanduser())).absolute()
            dir_name, filename = os.path.split(posix_path)
            if posix_path.exists() and posix_path.is_dir():
                dir_name = f"{dir_name}/{filename}"
                filename = ''
            if dir_name in ['.', '..', '']:
                dir_name = (dir_name or '.') + os.path.sep
            return PathObject(
                dir_name.strip(), relpath(dir_name, os.curdir), filename.strip(),
                posix_path.exists(), "folder" if posix_path.is_dir() else "file"
            )
        return None

    @staticmethod
    def split(path_name: AnyPath) -> tuple[str, str]:
        """Split any path into directory and file name."""
        po = PathObject.of(path_name)
        return po.abs_dir, po.filename

    def __str__(self) -> str:
        return self.join()

    def to_string(self) -> str:
        return f"PathObject: {json.dumps(self.__dict__, default=lambda obj: obj.__dict__)}"

    def join(self) -> str:
        """Split the path into directory and file names."""
        return join(self.abs_dir, self.filename)

    @property
    def is_dir(self) -> bool:
        return self.kind == 'folder'

    @property
    def is_file(self) -> bool:
        return self.kind == 'file'
